-- Einfache Pixel-Engine fr LCD-Grafiktreiber von E-Lab Pascal
-- Implementation von C. Meyer 10/2008

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
 
entity pixel_engine is 
  port (
	state_in			:in		std_logic;	-- state_out von Sync-Generator, 0=Video-Zyklus, 1=SPI-Zugriffs-Zyklus R/W
	data_write_in	:in		std_logic;	-- pos. Logik!
	color_in,cmd_in,
	mask_in,pixel_in,						-- Bytes vom E-Lab-LCD-Treiber
	byte_in			:in	std_logic_vector ( 7 downto 0);	-- gelesenes Byte aus VRAM
	count_up			:out	std_logic;	-- Adress-Inkrement nach Abschluss des Schreibzyklus
	write_out		:out	std_logic;	-- synchronisiertes und verzgertes Schreibsignal, zwei Zyklen lang
	wrsel_color		:out	std_logic;	-- ist high wenn Farb-Byte geschrieben wird, sonst low
	byte_out			:out	std_logic_vector ( 7 downto 0)	-- Daten-Byte ans VRAM
	);
end pixel_engine;
 
architecture behave of pixel_engine is

	signal data_write_del, data_write		:std_logic;
	signal set_wr, reset_wr		:std_logic;
	signal modify_state	:std_logic;
	
begin

wrsel_color <= modify_state;
 
process begin
	wait until falling_edge (state_in);
	data_write_del<=data_write_in;
	if (modify_state='1') then -- war vorher '0', Zyklus-Anfang
-- Pixel-Byte schreiben
		if (cmd_in=2) then
			if (pixel_in=0) then
				byte_Out <= (not mask_in);
			elsif (pixel_in=1) then
				byte_Out <= mask_in;
			elsif (pixel_in=2) then
				byte_Out <= byte_in xor mask_in;
			end if;
		elsif (cmd_in=4) then
			byte_Out <= byte_in or mask_in;
		elsif (cmd_in=5) then
			byte_Out <= byte_in and (not mask_in);
		elsif (cmd_in=6) then
			byte_Out <= byte_in xor mask_in;
		else 
			byte_Out <= mask_in;
		end if;
	else
-- Farb-Byte schreiben
		byte_Out <= color_in;
	end if;
-- Sequenz starten wenn SPI-bertragung abgeschlossen
	if data_write_del='0' then
-- asynchrones Write mit Byte-Clock synchronisieren
		set_wr <= '0';
		write_out <= data_write;
		count_up <= not data_write; -- steigende Flanke fr Adresszhler
		if data_write = '0' then
			modify_state <= '0';
		else
			modify_state <= not modify_state;
		end if;
		if modify_state = '1' then
			reset_wr <= '1';
		else
			reset_wr <= '0';
		end if;
	else
		set_wr <= '1';
		write_out <= '0';
		count_up <= '1'; -- steigende Flanke fr Adresszhler
	end if;
end process;


process(set_wr, reset_wr) begin
-- einfaches RS-Flipflop mit dominierendem Reset
   if reset_wr = '1' then -- reset hat Vorrang vor Reset
      data_write <= '0';
   elsif set_wr='1' then
      data_write <= '1';
   end if;
end process;


end behave;

