-- 6,25 MHz Input FBAS Clock Source fr c't FPGA, Implementation von C. Meyer 10/2008
-- QVGA 320 x 240 Pixel auf FBAS composite, s/w, RGB-Farbe durch color_sel-Byte im RAM, obere 16K
 
-- mit LineDoubler und 12,5 MHz Takt auch fr QVGA-Darstellung auf VGA-Monitor 31 kHz/60Hz

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
 
entity sync is 
  port (
	clock_in			:in	std_logic; -- Takt 12,5 MHz
	write_in			:in	std_logic; -- Write von Pixel_Engine
	wrsel_color		:in	std_logic; -- nur bei Farbe
	rw_adr 			:in	std_logic_vector (15 downto 0); -- Farbbyte wird automatisch in obere 16K gespeichert
	horiz_sync_out, vert_sync_out	:out	std_logic;
	vga_blank		:out	std_logic; -- Sync-Signale (active low), Blanking (Austastzeit low)
	OEV_n				:out	std_logic; -- Output Enable Video-Zyklus, active low
	OER_n				:out	std_logic; -- Output Enable SPI-Zugriffs-Zyklus, active low
	WE_n				:out	std_logic; -- Write Enable SPI-Zugriffs-Zyklus, active low
	state_out		:out	std_logic; -- Zyklus-Abschnitt, 0=Video-Zyklus, 1=SPI-Zugriffs-Zyklus R/W
	latch_color		:out	std_logic; -- D-FF-Clock zum Latchen des ausgelesenen Video-Farb-Bytes
	trist_data		:out	std_logic; -- Tristate VRAM-Data (1=high Z), )
	latch_pixels	:out	std_logic; -- Pixel-Schieberegister-Load
	sr_clock			:out	std_logic; -- Pixel-Schieberegister-Takt
	vram_adr 		:out	std_logic_vector (15 downto 0) -- bei QVGA nur 2x 9600 Bytes benutzt, 8 Pixel= 1 Byte bei s/w
	);
end sync;
 
architecture behave of sync is
	signal horiz_sync, vert_sync			:std_logic;
	signal video_on, video_on_v, video_on_h	:std_logic;
	signal WReq, WReq_pre			:std_logic;

	signal h_count, v_count			:integer range 0 to 1023;
	signal pixel_count				:std_logic_vector (18 downto 0);
	signal pixel_row, pixel_column	:integer range 0 to 1023;
	signal state_count				:std_logic_vector ( 2 downto 0);
	
-- Horizontal Timing Constants  
	CONSTANT H_pixels_across: 	Natural := 320;
	CONSTANT H_sync_low: 		Natural := 328;
	CONSTANT H_sync_high: 		Natural := 376;
	CONSTANT H_end_count: 		Natural := 400;
	
-- Vertical Timing Constants
	
-- Ein-/auskommentieren nach Bedarf:	
-- Composite 60 Hz:
--	CONSTANT V_pixels_down: 	Natural := 240;
--	CONSTANT V_sync_low: 		Natural := 245; -- NTSC 60 Hz
--	CONSTANT V_sync_high: 		Natural := 246; -- NTSC 60 Hz
--	CONSTANT V_end_count: 		Natural := 262; -- NTSC 60 Hz

-- Composite 50 Hz:
--	CONSTANT V_pixels_down: 	Natural := 240;
--	CONSTANT V_sync_low: 		Natural := 276; -- PAL 50 Hz
--	CONSTANT V_sync_high: 		Natural := 278; -- PAL 50 Hz
--	CONSTANT V_end_count: 		Natural := 312; -- PAL 50 Hz

-- QVGA 31 kHz/60Hz: LineDoubler muss eingeschaltet sein
	signal dbl_count					:integer range 0 to 1; -- nur bei QVGA LineDoubler bentigt
	CONSTANT V_pixels_down: 	Natural := 480;
	CONSTANT V_sync_low: 		Natural := 490;
	CONSTANT V_sync_high: 		Natural := 492;
	CONSTANT V_end_count: 		Natural := 525;

begin
  
 
video_on <= video_on_h and video_on_v;
vga_blank <= video_on;
sr_clock <= not clock_in;
Wreq<=  write_in;
 
process begin
	wait until (clock_in'event) and (clock_in='1');

	if (h_count = H_end_count) then
		h_count <= 0;
		state_count <= (others =>'0');
	else
		h_count <= h_count+1;
		state_count <= state_count + 1;
	end if;
	
	if (h_count <= H_sync_high) and (h_count >= H_sync_low) then
		horiz_sync <= '0';
	else
		horiz_sync <= '1';
	end if;
	
	
	if (v_count >= V_end_count) and (h_count >= H_sync_low) then
		v_count <= 0;
		dbl_count <= 0;					-- nur bei QVGA LineDoubler bentigt	
		pixel_count <= (others => '0');
	elsif (h_count = H_sync_low) then
		v_count <= v_count + 1;
		dbl_count <= dbl_count + 1;	-- nur bei QVGA LineDoubler bentigt
	end if;
 
	if (v_count <= V_sync_high) and (v_count >= V_sync_low) then
		vert_sync <= '0';
	else
  		vert_sync <= '1';
	end if;
	
	
	if (h_count < H_pixels_across) then
		video_on_h <= '1';
		pixel_column <= h_count;
-- nur die gltigen Pixel adressieren
   	pixel_count <= pixel_count+1;
	else
		video_on_h <= '0';
	end if;

-- Line Doubler, gleiche zeile nochmal bringen 
-- Auskommentieren fr FBAS/composite...
	if (h_count = H_pixels_across) and (dbl_count = 1) then
		pixel_count <= pixel_count-H_pixels_across;
	end if;
-- ...bis hierhin
 
	if (v_count <= V_pixels_down) then
		video_on_v <= '1';
		pixel_row <= v_count;
	else
		video_on_v <= '0';
	end if;
	
	horiz_sync_out <= horiz_sync; -- xnor vert_sync; -- fr FBAS einkommentieren
   vert_sync_out <= vert_sync;
	state_out <= state_count(2);
	

	if (state_count(2)='1') then
-- Write-Zyklus, 2. Hlfte von state_count
		OEV_n<='1';
		trist_data<= not Wreq;
	else
-- Lese-Zyklus, 1. Hlfte von state_count
		OEV_n<='0';
		trist_data<='1';
	end if;

-- Farb-Byte fr Video lesen
	if (state_count=0) or (state_count=1) then
		vram_adr <= (pixel_count(18 downto 3)) or b"0100_0000_0000_0000"; -- obere 16 KByte
-- Pixel-Byte fr Video lesen
	elsif (state_count=2) or (state_count=3) then
		latch_color <= '1';
		vram_adr <= pixel_count(18 downto 3);
	else
		latch_color <= '0';
-- Multiplexer fr Schreiben
		if wrsel_color = '1' then 
			vram_adr <= rw_adr or b"0100_0000_0000_0000"; -- obere 16 KByte
		else
			vram_adr <= rw_adr; -- obere 16 KByte;
		end if;
	end if;

-- Schieberegister laden
	if (state_count=3) then
		latch_pixels<='1';
	else
		latch_pixels<='0';
	end if;
	
-- Write-Zyklus, 2. Hlfte von state_count
	if (state_count=5) or (state_count=6) then
		OER_n<= Wreq;	-- wenn WriteRequest, bleibt OER_n high
		WE_n<= not Wreq;
	else
		OER_n<='1';
		WE_n<= '1';
	end if;
		
end process;
	

end behave;

